import { Point } from './Node';
import NodeFixedBounds from './NodeFixedBounds';
export interface ArcStyle {
    radius: number;
    lineWidth?: number;
    strokeStyle?: string;
    fillStyle?: string;
    lineDash?: Array<number>;
    closed?: boolean;
}
export interface ArcParameters extends ArcStyle, Partial<Point> {
    startAngle: number;
    endAngle: number;
    counterClockwise?: boolean;
}
declare class Arc<T> extends NodeFixedBounds<T> {
    protected x: number;
    protected y: number;
    protected radius: number;
    protected lineWidth: number;
    protected strokeStyle: string;
    protected fillStyle: string;
    protected lineDash: Array<number>;
    protected startAngle: number;
    protected endAngle: number;
    protected closed: boolean;
    protected counterClockwise: boolean;
    constructor({x, y, radius, lineWidth, strokeStyle, fillStyle, lineDash, startAngle, endAngle, closed, counterClockwise}: ArcParameters);
    draw(context: CanvasRenderingContext2D): void;
    intersection({x, y}: Point): Arc<T> | undefined;
    private normalizeAngle(angle);
    private isAngleBetween(angle, start, end);
    private deltaAngles(start, end);
    private isCircle();
}
export default Arc;
