"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const NodeFixedBounds_1 = require("./NodeFixedBounds");
class Line extends NodeFixedBounds_1.default {
    constructor({ x1, y1, x2, y2, strokeStyle = 'rgba(0, 0, 0, 1)', lineWidth = 1, lineCap = 'butt', lineDash = [] }) {
        const minX = Math.min(x1, x2) - lineWidth / 2;
        const minY = Math.min(y1, y2) - lineWidth / 2;
        const maxX = Math.max(x1, x2) + lineWidth / 2;
        const maxY = Math.max(y1, y2) + lineWidth / 2;
        const bounds = { minX, minY, maxX, maxY };
        super(bounds);
        this.x1 = x1;
        this.y1 = y1;
        this.x2 = x2;
        this.y2 = y2;
        this.strokeStyle = strokeStyle;
        this.lineWidth = lineWidth;
        this.lineCap = lineCap;
        this.lineDash = lineDash;
    }
    draw(context) {
        const oldStrokeStyle = context.strokeStyle;
        const oldLineWidth = context.lineWidth;
        const oldLineCap = context.lineCap;
        const oldLineDash = context.getLineDash();
        context.strokeStyle = this.strokeStyle;
        context.lineWidth = this.lineWidth;
        context.lineCap = this.lineCap;
        context.beginPath();
        context.moveTo(this.x1, this.y1);
        context.lineTo(this.x2, this.y2);
        if (this.lineWidth > 0) {
            context.setLineDash(this.lineDash);
            context.stroke();
        }
        context.closePath();
        context.strokeStyle = oldStrokeStyle;
        context.lineWidth = oldLineWidth;
        context.lineCap = oldLineCap;
        if (this.lineWidth > 0) {
            context.setLineDash(oldLineDash);
        }
    }
    intersection({ x, y }) {
        const distance2 = Math.pow((this.x2 - this.x1), 2) + Math.pow((this.y2 - this.y1), 2);
        let t;
        if (distance2 === 0) {
            t = 0;
        }
        else {
            t = ((x - this.x1) * (this.x2 - this.x1) + (y - this.y1) * (this.y2 - this.y1)) / distance2;
        }
        if (0 <= t && t <= 1) {
            const dx = x - (this.x1 + t * (this.x2 - this.x1));
            const dy = y - (this.y1 + t * (this.y2 - this.y1));
            if (Math.pow(dx, 2) + Math.pow(dy, 2) <= Math.pow((this.lineWidth / 2), 2)) {
                return this;
            }
        }
    }
}
exports.default = Line;
//# sourceMappingURL=Line.js.map